<?php
namespace Pinpoint\Conesso\Console\Command\Orders;

use Pinpoint\Conesso\Model\Api\Management\Process\ProcessFactory;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class  Sync Orders Command
 */
class SyncOrdersCommand extends Command
{
    const BATCH_SIZE = 'batch_size';
    const STORE_ID = 'store_id';

    /**
     * @var ProcessFactory
     */
    private ProcessFactory $processFactory;

    /**
     * @param ProcessFactory $processFactory
     */
    public function __construct(
        ProcessFactory $processFactory
    ) {
        $this->processFactory = $processFactory;
        parent::__construct();
    }

    /**
     * @inheritDoc
     */
    protected function configure(): void
    {
        $this->setName('conesso:sync:orders');
        $this->setDescription('Forced running of all pending events in the conesso order table.');

        $this->addOption(
            self::BATCH_SIZE,
            null,
            InputOption::VALUE_OPTIONAL,
            'Batch Size'
        );

        $this->addOption(
            self::STORE_ID,
            null,
            InputOption::VALUE_OPTIONAL,
            'Store Id'
        );

        parent::configure();
    }

    /**
     * Execute the command
     *
     * @param InputInterface $input
     * @param OutputInterface $output
     *
     * @return int
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $this->processFactory->getProcess('order')->synchronizeData();
        $output->writeln('<info>Success Message.</info>');
        return 0;
    }
}
