<?php
/**
 *
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Pinpoint\Conesso\Controller\Adminhtml\Connection;

use Exception;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Exception\LocalizedException;
use Pinpoint\Conesso\Model\Api\Access\Auth;

class TestConnection extends Action
{
    /**
     * @var JsonFactory
     */
    private JsonFactory $resultJsonFactory;

    /**
     * @var Auth
     */
    private Auth $auth;

    /**
     * @param Context $context
     * @param Auth $auth
     * @param JsonFactory $resultJsonFactory
     */
    public function __construct(
        Context $context,
        Auth $auth,
        JsonFactory $resultJsonFactory
    ) {
        parent::__construct($context);
        $this->auth = $auth;
        $this->resultJsonFactory = $resultJsonFactory;
    }

    /**
     * Check for connection to server
     *
     * @return Json
     */
    public function execute(): Json
    {
        $result = [
            'success' => false,
            'errorMessage' => '',
        ];
        $options = $this->getRequest()->getParams();
        try {
            $connectionType = $options['connection_type'];
            $apiKey = $connectionType === '1' ? $options['test_api_key'] : $options['production_api_key'];
            $response = $this->auth->testConnection($connectionType, $apiKey);
            if ($response) {
                $result['success'] = true;
            }
        } catch (LocalizedException|Exception $e) {
            $result['success'] = false;
        }

        $resultJson = $this->resultJsonFactory->create();
        return $resultJson->setData($result);
    }
}
