<?php

namespace Pinpoint\Conesso\Cron;

use Magento\Framework\App\Cache\Type\Config;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Pinpoint\Conesso\Model\Api\Management\Process\ProcessFactory;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Pinpoint\Conesso\Model\Config\OrderSettings;

class Order
{
    /**
     * @var ProcessFactory
     */
    private ProcessFactory $processFactory;

    /**
     * @var WriterInterface
     */
    private WriterInterface $configWriter;

    /**
     * @var DateTime
     */
    private DateTime $dateTime;

    /**
     * @var TypeListInterface
     */
    private TypeListInterface $cacheTypeList;

    /**
     * @var OrderSettings
     */
    private OrderSettings $orderSettings;

    /**
     * @param ProcessFactory $processFactory
     * @param OrderSettings $orderSettings
     * @param WriterInterface $configWriter
     * @param DateTime $dateTime
     * @param TypeListInterface $cacheTypeList
     */
    public function __construct(
        ProcessFactory  $processFactory,
        OrderSettings $orderSettings,
        WriterInterface $configWriter,
        DateTime $dateTime,
        TypeListInterface $cacheTypeList
    )
    {
        $this->processFactory = $processFactory;
        $this->orderSettings = $orderSettings;
        $this->configWriter = $configWriter;
        $this->dateTime = $dateTime;
        $this->cacheTypeList = $cacheTypeList;
    }

    /**
     * @return void
     */
    public function execute(): void
    {
        if ($this->orderSettings->isModuleEnabled() && $this->orderSettings->isCronEnabled()) {
            if ($this->processFactory->getProcess('order')->synchronizeData()) {
                $this->configWriter->save(OrderSettings::XML_PATH_LAST_CRON_RUN_TIMESTAMP, $this->dateTime->gmtDate());
                $this->cacheTypeList->cleanType(Config::TYPE_IDENTIFIER);
            }
        }
    }
}
