<?php
namespace Pinpoint\Conesso\Model\Api\Access;

use Conesso;
use GuzzleHttp\Client;
use Pinpoint\Conesso\Model\Config\BasicSettings;
use Throwable;

class Auth
{
    const TEST_BASE_URI = 'https://api.qa.conesso.io/v2';
    const PRODUCTION_BASE_URI = 'https://api.conesso.io/v2'; // TODO: CHANGE TO THE CORRECT ONE
    const TEST_CONNECTION_TYPE_ID = '1';
    const PRODUCTION_CONNECTION_TYPE_ID = '2';

    /**
     * @var BasicSettings
     */
    private BasicSettings $config;

    /**
     * @var Client
     */
    private Client $httpClient;

    /**
     * @param BasicSettings $config
     */
    public function __construct(
        BasicSettings $config
    ) {
        $this->config = $config;
        $this->httpClient = new Client();
    }

    /**
     * Get connection client based on the connection type /test/production
     * @return Conesso\Client
     */
    public function getConnessoClient(): Conesso\Client
    {
        $connectionType = $this->config->getConnectionType();
        $apiKey = $this->getApiKeyByConnectionType($connectionType);
        $baseUri = $this->getBaseUri($connectionType);
        return Conesso::factory()
            ->withApiKey($apiKey)
            ->withHttpHeader('User-Agent', 'MyApp/1.0')
            ->withBaseUri($baseUri)
            ->withHttpClient($this->httpClient)
            ->make();
    }

    /**
     * Test connection from Basic Configuration "Test Connection button"
     * @param $connectionType
     * @param $apiKey
     * @return array|false
     */
    public function testConnection($connectionType, $apiKey): bool|array
    {
        $baseUri = $this->getBaseUri($connectionType);
        try {
            return Conesso::factory()
                ->withApiKey($apiKey)
                ->withHttpHeader('User-Agent', 'MyApp/1.0')
                ->withBaseUri($baseUri)
                ->withHttpClient($this->httpClient)
                ->make()->contacts()->list()->toArray();
        } catch (Throwable $ignored) {
            return false;
        }
    }

    /**
     * @param $connectionType
     * @return mixed
     */
    public function getApiKeyByConnectionType($connectionType): mixed
    {
        if($connectionType === self::TEST_CONNECTION_TYPE_ID) {
            return $this->config->getTestApiKey();
        }
        return $this->config->getProductionApiKey();
    }

    /**
     * Get Base Uri based on connection type test/live
     * @param $connectionType
     * @return string
     */
    protected function getBaseUri($connectionType): string
    {
        if($connectionType === self::TEST_CONNECTION_TYPE_ID) {
            return self::TEST_BASE_URI;
        }

        return self::PRODUCTION_BASE_URI;
    }
}
