<?php
namespace Pinpoint\Conesso\Model\Api\Repository;

use Exception;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\SortOrder;
use Magento\Framework\Model\AbstractModel;
use Pinpoint\Conesso\Api\Data\CartSyncInterface;
use Pinpoint\Conesso\Api\Repository\CartsSyncRepositoryInterface;
use Pinpoint\Conesso\Model\ResourceModel\SyncCarts\CollectionFactory;
use Pinpoint\Conesso\Model\ResourceModel\SyncCarts as SyncCartsResourceModel;
use Pinpoint\Conesso\Api\Data\CartSyncInterfaceFactory;
use Magento\Framework\Exception\CouldNotSaveException;
use Pinpoint\Conesso\Model\SyncCarts;

class CartsSyncRepository implements CartsSyncRepositoryInterface
{
    /**
     * @var CollectionFactory
     */
    private CollectionFactory $syncCartsCollectionFactory;

    /**
     * @var CartSyncInterfaceFactory
     */
    private CartSyncInterfaceFactory $cartSyncFactory;

    /**
     * @var SyncCartsResourceModel
     */
    private SyncCartsResourceModel $syncCartsResourceModel;

    /**
     * @param CollectionFactory $syncCartsCollectionFactory
     * @param SyncCartsResourceModel $syncCartsResourceModel
     * @param CartSyncInterfaceFactory $cartSyncFactory
     */
    public function __construct(
        CollectionFactory $syncCartsCollectionFactory,
        SyncCartsResourceModel $syncCartsResourceModel,
        CartSyncInterfaceFactory $cartSyncFactory
    ) {
        $this->cartSyncFactory = $cartSyncFactory;
        $this->syncCartsResourceModel = $syncCartsResourceModel;
        $this->syncCartsCollectionFactory = $syncCartsCollectionFactory;
    }

    /**
     * @param CartSyncInterface $cartSync
     * @return CartSyncInterface
     * @throws CouldNotSaveException
     */
    public function save(CartSyncInterface $cartSync)
    {
        try {
            /** @var SyncCarts|AbstractModel $wishList */
            $this->syncCartsResourceModel->save($cartSync);
            $x = $cartSync;
            return $cartSync;
        } catch (Exception $e) {
            throw new CouldNotSaveException(__(
                'Could not save the entity: %1',
                $e->getMessage()
            ));
        }
    }

    /**
     * @param $cartId
     * @return mixed
     * @throws CouldNotSaveException
     */
    public function getById($cartId): mixed
    {
        try {
            return $this->syncCartsCollectionFactory->create()->addFieldToFilter('cart_id', $cartId)->getFirstItem();
        } catch (Exception $e) {
            throw new CouldNotSaveException(__(
                'Entity not found! %1',
                $e->getMessage()
            ));
        }
    }

    /**
     * @param CartSyncInterface $cartSync
     * @return bool
     * @throws Exception
     */
    public function delete(CartSyncInterface $cartSync): bool
    {
        $this->syncCartsResourceModel->delete($cartSync);
        return true;
    }

    /**
     * @param int $id
     * @return bool
     */
    public function deleteById(int $id): bool
    {
        try {
            $syncCarts = $this->syncCartsCollectionFactory->create()
                ->addFieldToFilter('id', ['in' => $id]);
            $syncCarts->walk('delete');
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(SearchCriteriaInterface $searchCriteria): mixed
    {
        $collection = $this->syncCartsCollectionFactory->create();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }
        $sortOrdersData = $searchCriteria->getSortOrders();
        if ($sortOrdersData) {
            foreach ($sortOrdersData as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        return $collection->getItems();
    }
}
