<?php
namespace Pinpoint\Conesso\Model\Api\Repository;

use Exception;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\SortOrder;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Pinpoint\Conesso\Api\Data\ContactSyncInterface;
use Pinpoint\Conesso\Api\Repository\ContactsRepositoryInterface;
use Pinpoint\Conesso\Model\ResourceModel\SyncContacts as SyncContactsResourceModel;
use Pinpoint\Conesso\Model\ResourceModel\SyncContacts\CollectionFactory;

class ContactRepository implements ContactsRepositoryInterface
{
    /**
     * @var CollectionFactory
     */
    private CollectionFactory $syncContactsCollectionFactory;

    /**
     * @var SyncContactsResourceModel
     */
    private SyncContactsResourceModel $syncContactsResourceModel;

    /**
     * @var AdapterInterface
     */
    private AdapterInterface $connection;

    /**
     * @param CollectionFactory $syncContactsCollectionFactory
     * @param SyncContactsResourceModel $syncContactsResourceModel
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        CollectionFactory $syncContactsCollectionFactory,
        SyncContactsResourceModel $syncContactsResourceModel,
        ResourceConnection $resourceConnection
    ) {
        $this->syncContactsResourceModel = $syncContactsResourceModel;
        $this->syncContactsCollectionFactory = $syncContactsCollectionFactory;
        $this->connection = $resourceConnection->getConnection();
    }

    /**
     * @param ContactSyncInterface $contactSync
     * @return int|mixed
     * @throws CouldNotSaveException
     */
    public function save(ContactSyncInterface $contactSync): mixed
    {
        try {
            return $this->connection->insertOnDuplicate(
                ContactSyncInterface::SCHEMA,
                $contactSync->getData()
            );
        } catch (Exception $e) {
            throw new CouldNotSaveException(__(
                'Could not save the entity: %1',
                $e->getMessage()
            ));
        }
    }

    /**
     * @param ContactSyncInterface $contactSync
     * @return bool
     * @throws Exception
     */
    public function delete(ContactSyncInterface $contactSync): bool
    {
        $this-> syncContactsResourceModel->delete($contactSync);
        return true;
    }

    /**
     * @param int $id
     * @return bool
     */
    public function deleteById(int $id): bool
    {
        try {
            $syncOrders = $this->syncContactsCollectionFactory->create()
                ->addFieldToFilter('id', ['in' => $id]);
            $syncOrders->walk('delete');
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(SearchCriteriaInterface $searchCriteria): mixed
    {
        $collection = $this->syncContactsCollectionFactory->create();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }
        $sortOrdersData = $searchCriteria->getSortOrders();
        if ($sortOrdersData) {
            foreach ($sortOrdersData as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        return $collection->getItems();
    }
}
