<?php
namespace Pinpoint\Conesso\Model\Api\Repository;

use Exception;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Model\AbstractModel;
use Pinpoint\Conesso\Api\Data\OrderSyncInterface;
use Pinpoint\Conesso\Api\Repository\OrdersSyncRepositoryInterface;
use Pinpoint\Conesso\Model\ResourceModel\SyncOrders\CollectionFactory;
use Pinpoint\Conesso\Model\ResourceModel\SyncOrders as SyncOrdersResourceModel;
use Pinpoint\Conesso\Api\Data\OrderSyncInterfaceFactory;
use Magento\Framework\Exception\CouldNotSaveException;
use Pinpoint\Conesso\Model\SyncOrders;

class OrdersSyncRepository implements OrdersSyncRepositoryInterface
{
    /**
     * @var CollectionFactory
     */
    private CollectionFactory $syncOrdersCollectionFactory;

    /**
     * @var OrderSyncInterfaceFactory
     */
    private OrderSyncInterfaceFactory $orderSyncFactory;

    /**
     * @var SyncOrdersResourceModel
     */
    private SyncOrdersResourceModel $syncOrdersResourceModel;

    /**
     * @param CollectionFactory $syncOrdersCollectionFactory
     * @param SyncOrdersResourceModel $syncOrdersResourceModel
     * @param OrderSyncInterfaceFactory $orderSyncFactory
     */
    public function __construct(
        CollectionFactory $syncOrdersCollectionFactory,
        SyncOrdersResourceModel $syncOrdersResourceModel,
        OrderSyncInterfaceFactory $orderSyncFactory
    ) {
        $this->orderSyncFactory = $orderSyncFactory;
        $this->syncOrdersResourceModel = $syncOrdersResourceModel;
        $this->syncOrdersCollectionFactory = $syncOrdersCollectionFactory;
    }

    /**
     * @param OrderSyncInterface $orderSync
     * @return OrderSyncInterface
     * @throws CouldNotSaveException
     */
    public function save(OrderSyncInterface $orderSync)
    {
        try {
            /** @var SyncOrders|AbstractModel $wishList */
            $this->syncOrdersResourceModel->save($orderSync);
            $x = $orderSync;
            return $orderSync;
        } catch (Exception $e) {
            throw new CouldNotSaveException(__(
                'Could not save the entity: %1',
                $e->getMessage()
            ));
        }
    }

    /**
     * @param OrderSyncInterface $orderSync
     * @return bool
     * @throws Exception
     */
    public function delete(OrderSyncInterface $orderSync): bool
    {
        $this->syncOrdersResourceModel->delete($orderSync);
        return true;
    }

    /**
     * @param int $id
     * @return bool
     */
    public function deleteById(int $id): bool
    {
        try {
            $syncOrders = $this->syncOrdersCollectionFactory->create()
                ->addFieldToFilter('id', ['in' => $id]);
            $syncOrders->walk('delete');
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(SearchCriteriaInterface $searchCriteria): mixed
    {
        $collection = $this->syncOrdersCollectionFactory->create();
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        return $collection->getItems();
    }
}
