<?php
namespace Pinpoint\Conesso\Model\Api\Repository;

use Exception;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Pinpoint\Conesso\Api\Data\ProductSyncInterface;
use Pinpoint\Conesso\Api\Repository\ProductsSyncRepositoryInterface;
use Pinpoint\Conesso\Model\ResourceModel\SyncProducts\CollectionFactory;
use Pinpoint\Conesso\Model\ResourceModel\SyncProducts as SyncProductsResourceModel;
use Magento\Framework\Api\SortOrder;
use Magento\Framework\App\ResourceConnection;

class ProductsRepository implements ProductsSyncRepositoryInterface
{
    /**
     * @var CollectionFactory
     */
    private CollectionFactory $productSyncCollectionFactory;

    /**
     * @var SyncProductsResourceModel
     */
    private SyncProductsResourceModel $resourceModel;

    /**
     * @var AdapterInterface
     */
    private AdapterInterface $connection;

    /**
     * @param CollectionFactory $productSyncCollectionFactory
     * @param SyncProductsResourceModel $resourceModel
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        CollectionFactory $productSyncCollectionFactory,
        SyncProductsResourceModel $resourceModel,
        ResourceConnection $resourceConnection
    ) {
        $this->productSyncCollectionFactory = $productSyncCollectionFactory;
        $this->resourceModel = $resourceModel;
        $this->connection = $resourceConnection->getConnection();

    }

    /**
     * @param ProductSyncInterface $productSync
     * @return int
     * @throws CouldNotSaveException
     */
    public function save(ProductSyncInterface $productSync)
    {
        try {
            $x = $this->connection->insertOnDuplicate(
                ProductSyncInterface::SCHEMA,
                $productSync->getData()
            );
            return $x;
        } catch (Exception $e) {
            throw new CouldNotSaveException(__(
                'Could not save the entity: %1',
                $e->getMessage()
            ));
        }

    }

    /**
     * @param ProductSyncInterface $productSync
     * @return bool
     * @throws Exception
     */
    public function delete(ProductSyncInterface $productSync): bool
    {
        $this->resourceModel->delete($productSync);
        return true;
    }

    /**
     * @param int $id
     * @return bool
     */
    public function deleteById(int $id): bool
    {
        try {
            $syncProducts = $this->productSyncCollectionFactory->create();
            if($id)
                $syncProducts->addFieldToFilter('id', ['in' => $id]);
            $syncProducts->walk('delete');
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return mixed
     */
    public function getList(SearchCriteriaInterface $searchCriteria): mixed
    {
        $collection = $this->productSyncCollectionFactory->create();
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }
        $sortOrdersData = $searchCriteria->getSortOrders();
        if ($sortOrdersData) {
            foreach ($sortOrdersData as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        return $collection->getItems();
    }
}
