<?php
namespace Pinpoint\Conesso\Model\Config;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;

class CartSettings
{
    const XML_PATH_IS_CART_SYNC_ENABLED = 'abandoned_carts/settings/enabled';
    const XML_PATH_IS_CRON_ENABLED = 'abandoned_carts/settings/cron_schedule';
    const XML_PATH_SYNC_FROM_DATE = 'abandoned_carts/settings/sync_from';
    const XML_PATH_BATCH_SIZE = 'abandoned_carts/settings/batch_size';
    const XML_PATH_LAST_CRON_RUN_TIMESTAMP = 'abandoned_carts/settings/last_run_timestamp';

    /**
     * @var ScopeConfigInterface
     */
    protected ScopeConfigInterface $scopeConfig;

    /**
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Is module enabled
     * @return string
     */
    public function isModuleEnabled(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_IS_CART_SYNC_ENABLED,
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function isCronEnabled(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_IS_CRON_ENABLED,
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getSyncFromDate(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_SYNC_FROM_DATE,
            ScopeInterface::SCOPE_STORE
        ) ?? false;
    }

    /**
     * @return string
     */
    public function getBatchSize(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_BATCH_SIZE,
            ScopeInterface::SCOPE_STORE
        );
    }
}
