<?php
namespace Pinpoint\Conesso\Model\Config;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;

class OrderSettings
{
    const XML_PATH_IS_PRODUCT_SYNC_ENABLED = 'order_sync_settings/order_sync_group/enabled';
    const XML_PATH_IS_CRON_ENABLED = 'order_sync_settings/order_sync_group/cron_schedule';
    const XML_PATH_SYNC_FROM_DATE = 'order_sync_settings/order_sync_group/sync_from';
    const XML_PATH_CUSTOM_ATTRIBUTES = 'order_sync_settings/order_sync_group/custom_attributes';
    const XML_PATH_BATCH_SIZE = 'order_sync_settings/order_sync_group/batch_size';
    const XML_PATH_LAST_CRON_RUN_TIMESTAMP = 'order_sync_settings/order_sync_group/last_run_timestamp';

    /**
     * @var ScopeConfigInterface
     */
    protected ScopeConfigInterface $scopeConfig;

    /**
     * @var Json
     */
    private Json $serialize;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param Json $serialize
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Json $serialize
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->serialize = $serialize;
    }

    /**
     * Is module enabled
     * @return string
     */
    public function isModuleEnabled(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_IS_PRODUCT_SYNC_ENABLED,
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function isCronEnabled(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_IS_CRON_ENABLED,
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string|bool
     */
    public function getSyncFromDate(): string|bool
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_SYNC_FROM_DATE,
            ScopeInterface::SCOPE_STORE
        ) ?? false;
    }

    /**
     * @return array|false
     */
    public function getCustomAttributes()
    {
        $attributesConfig = $this->scopeConfig->getValue(self::XML_PATH_CUSTOM_ATTRIBUTES, ScopeInterface::SCOPE_STORE);
        if ($attributesConfig == '' || $attributesConfig == null) {
            return false;
        }

        $unserialized = $this->serialize->unserialize($attributesConfig);
        $attributesArray = [];
        foreach ($unserialized as $key => $row) {
            $attributesArray[] = [
                'magento_attribute' => $row['magento_attribute'],
                'conesso_attribute' => $row['conesso_attribute']
            ];
        }

        return $attributesArray;
    }

    /**
     * @return string
     */
    public function getBatchSize(): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_BATCH_SIZE,
            ScopeInterface::SCOPE_STORE
        );
    }
}
